package CruiserHard.Drawing;

import CruiserHard.Direction.Direction;
import CruiserHard.Entity.EntityCruiser;

import javax.swing.*;
import java.awt.*;
import java.awt.geom.Ellipse2D;
import java.util.*;

public class DrawingCruiser extends JPanel {
    public EntityCruiser cruiser; //Класс-сущность
    public int _startPosX; //Координаты отрисовки по оси x
    public EntityCruiser GetCruiser(){return cruiser;}
    public int GetStartX() {
        return _startPosX;
    }
    public int _startPosY; //Координаты отрисовки по оси y
    public int GetStartY() {
        return _startPosY;
    }
    private Integer _pictureWidth = null; //Ширина окна
    private Integer _pictureHeight = null; //Высота окна
    private final int _cruiserWidth = 150; //Ширина отрисовки крейсера
    public int GetWidth() {
        return _cruiserWidth;
    }
    private final int _cruiserHeight = 60; //Высота отрисовки крейсера
    public int GetHeight() {
        return _cruiserHeight;
    }

    //Инициализация
    public DrawingCruiser(int speed, float weight, Color bodyColor, int width, int height, int blockCount) {
        cruiser = new EntityCruiser(speed,weight,bodyColor);
        _pictureWidth = width;
        _pictureHeight = height;

    }

    //Начальные коордитанты
    public void SetPosition(int x, int y, int width, int height)
    {
        if (width < _cruiserWidth || height < _cruiserHeight) return;
        Random random = new Random();
        _startPosX = x < 0 || x + _cruiserWidth > width ? random.nextInt(0, width - _cruiserWidth) : x;
        _startPosY = y < 0 || y + _cruiserHeight > height ? random.nextInt(0, height - _cruiserHeight) : y;
        _pictureWidth = width;
        _pictureHeight = height;
    }

    //Движение транспорта по координатам
    public void MoveTransport(Direction direction)
    {
        if (_pictureWidth == null || _pictureHeight == null) return;
        switch (direction)
        {
            case Left: //Влево
                if (_startPosX - cruiser.GetStep() > 0) _startPosX -= cruiser.GetStep();
                break;
            case Up: //Вверх
                if (_startPosY - cruiser.GetStep() > 0) _startPosY -= cruiser.GetStep();
                break;
            case Right: //Вправо
                if (_startPosX + _cruiserWidth + cruiser.GetStep() < _pictureWidth) _startPosX += cruiser.GetStep();
                break;
            case Down: //Вниз
                if (_startPosY + _cruiserHeight + cruiser.GetStep() < _pictureHeight) _startPosY += cruiser.GetStep();
                break;
        }
    }

    //Отрисовка транспорта
    public void DrawTransport(Graphics gr)
    {
        if (GetCruiser() == null) return;

        if (_startPosX < 0 || _startPosY < 0 || _pictureWidth == null || _pictureHeight == null)
        {
            return;
        }
        super.paintComponent(gr);
        Graphics2D g2d = (Graphics2D) gr;
        g2d.setColor(cruiser.GetBodyColor());
        // палуба
        Polygon paluba = new Polygon();
        paluba.addPoint(_startPosX + 10,_startPosY);
        paluba.addPoint(_startPosX + 110,_startPosY);
        paluba.addPoint(_startPosX + 160,_startPosY + 30);
        paluba.addPoint(_startPosX + 110,_startPosY + 60);
        paluba.addPoint(_startPosX + 10,_startPosY + 60);
        g2d.fillPolygon(paluba);
        // элементы
        Polygon elements = new Polygon();
        elements.addPoint(_startPosX + 50,_startPosY + 20);
        elements.addPoint(_startPosX + 70,_startPosY + 20);
        elements.addPoint(_startPosX + 70,_startPosY + 10);
        elements.addPoint(_startPosX + 90,_startPosY + 10);
        elements.addPoint(_startPosX + 90,_startPosY + 50);
        elements.addPoint(_startPosX + 70,_startPosY + 50);
        elements.addPoint(_startPosX + 70,_startPosY + 40);
        elements.addPoint(_startPosX + 50,_startPosY + 40);
        g2d.setColor(Color.BLACK);
        g2d.fillPolygon(elements);
        g2d.fill(new Ellipse2D.Double(_startPosX + 100, _startPosY + 20, 20, 20));
        // турбины
        g2d.setColor(Color.BLACK);
        g2d.fillRect(_startPosX, _startPosY + 10, 10, 20);
        g2d.fillRect(_startPosX, _startPosY + 35, 10, 20);

    }

    //Изменение границ отрисовки
    public boolean CanMove(Direction direction) {
        if (cruiser == null) {
            return false;
        }
        return switch (direction) {
            case Left -> _startPosX - cruiser.GetStep() > 0;
            case Up -> _startPosY - cruiser.GetStep() > 0;
            case Right -> _startPosX + _cruiserWidth + cruiser.GetStep() < _pictureWidth;
            case Down -> _startPosY + _cruiserHeight + cruiser.GetStep() < _pictureHeight;
        };
    }
}