package CruiserHard.MovementStrategy;;
import CruiserHard.Direction.*;
public abstract class AbstractStrategy {
    private IMoveableObject moveableObject;

    private Status state = Status.NotInit;

    private int fieldWidth;

    protected int GetFieldWidth() {
        return fieldWidth;
    }

    private int fieldHeight;

    protected int GetFieldHeight() {
        return fieldHeight;
    }

    public Status GetStatus() {
        return state;
    }

    public void SetData(IMoveableObject moveableObject, int width, int height) {
        if (moveableObject == null) {
            state = Status.NotInit;
            return;
        }
        state = Status.InProgress;
        this.moveableObject = moveableObject;
        fieldWidth = width;
        fieldHeight = height;
    }

    public void MakeStep() {
        if (state != Status.InProgress) {
            return;
        }
        if (isTargetDestination()) {
            state = Status.Finish;
            return;
        }
        MoveToTarget();
    }

    protected boolean MoveLeft() {
        return MoveTo(Direction.Left);
    }

    protected boolean MoveRight() {
        return MoveTo(Direction.Right);
    }

    protected boolean MoveUp() { return MoveTo(Direction.Up); }

    protected boolean MoveDown() {
        return MoveTo(Direction.Down);
    }

    protected ObjectParameters GetObjectParameters() {
        if (moveableObject == null) {
            return null;
        }
        return moveableObject.GetObjectPosition();
    }

    protected Integer  GetStep() {
        if (state != Status.InProgress) {
            return null;
        }
        return moveableObject.GetStep();
    }

    protected abstract void MoveToTarget();

    protected abstract boolean isTargetDestination();

    private boolean MoveTo(Direction direction) {
        if (state != Status.InProgress) {
            return false;
        }
        if (moveableObject == null) {
            return false;
        }
        if (moveableObject.CheckCanMove(direction)) {
            moveableObject.MoveObject(direction);
            return true;
        }
        return false;
    }
}
